<?php

namespace ByDedi\Module\CustomForm\Model;

use ByDedi\Module\CustomForm\Service\NoIdException;
use LogicException;

/**
 * Class FormResponse
 *
 * @property int $form_response_id
 */
class FormResponse extends Model implements FormResponseInterface
{
    /**
     * @var string
     */
    protected $table = 'form_response';

    /**
     * @var FormEntityInterface
     */
    private $form;

    /**
     * @var FieldResponse[]
     */
    private $fields = [];

    public function __construct(FormEntityInterface $formEntity)
    {
        parent::__construct();

        $this->form = $formEntity;
    }

    /**
     * {@inheritdoc}
     */
    public function getId()
    {
        if (!$this->hasId()) {
            throw new NoIdException();
        }
        return $this->form_response_id;
    }

    /**
     * {@inheritdoc}
     * @return FieldResponse[]
     */
    public function getFields()
    {
        return $this->fields;
    }

    /**
     * {@inheritdoc}
     * @throws LogicException
     */
    public function save($data = null)
    {
        $formId = $this->getForm()->getId();
        if (empty($formId)) {
            throw new LogicException("Form must have an id to be saved as a response");
        }
        $this->form_id = $formId;

        $formResponseId = parent::save($data);
        $this->form_response_id = $formResponseId;

        $this->saveFields();

        return $this->getId();
    }

    /**
     * @param FieldResponse[] $fields
     */
    public function addFields(FieldResponse ...$fields)
    {
        array_walk($fields, function (FieldResponse $field) {
            if ($this->hasField($field)) {
                try {
                    $fieldId = $field->getId();
                    trigger_error(
                        "A FormResponseField with the id '{$fieldId}' is already present"
                        . " and it has not been added in " . __METHOD__
                    );
                } catch (NoIdException $exception) {
                    $class = __CLASS__;
                    throw new LogicException(
                        "$class::hasField() returned true for a field with no id",
                        $exception->getCode(),
                        $exception
                    );
                }
                return;
            }

            $this->fields[] = $field;
        });
    }

    /**
     * @param FieldResponse $fieldToAdd
     * @return bool
     */
    public function hasField(FieldResponse $fieldToAdd)
    {
        if (!$fieldToAdd->hasId()) {
            return false;
        }

        return array_reduce(
            $this->getFields(),
            function ($hasField, FieldResponse $potentiallyDuplicate) use ($fieldToAdd) {
                if ($hasField) {
                    return true;
                }

                if (!$potentiallyDuplicate->hasId()) {
                    return false;
                }

                return $fieldToAdd->getId() === $potentiallyDuplicate->getId();
            },
            false
        );
    }

    /**
     * @return FormEntityInterface
     */
    public function getForm()
    {
        return $this->form;
    }

    /**
     * @return bool
     */
    public function hasId()
    {
        return $this->form_response_id !== null;
    }

    private function saveFields()
    {
        $formFieldResponses = $this->getFields();
        array_walk($formFieldResponses, function (FieldResponse $field) {
            $field->save();
        });
    }
}
