<?php

namespace ByDedi\Module\CustomForm\Service;

use ByDedi\Module\CustomForm\Model\CustomConstraintInterface;
use ByDedi\Module\CustomForm\Model\FieldEntityInterface;
use Form\FullForm\FieldInterface;
use Form\FullForm\FieldTrait;
use Form\FullForm\OptionInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class FilledField
 */
class FilledFieldFromEntity implements FieldFromEntityInterface
{
    use FieldTrait;

    /**
     * @var FieldFromEntityInterface
     */
    private $field;

    /**
     * @var Request
     */
    private $request;

    /**
     * FilledField constructor.
     * @param FieldFromEntityInterface $field
     * @param Request $request
     */
    public function __construct(FieldFromEntityInterface $field, Request $request)
    {
        $this->field = $field;
        $this->request = $request;
    }

    /**
     * {@inheritdoc}
     */
    public function getFieldEntity()
    {
        return $this->getField()->getFieldEntity();
    }

    /**
     * @return FieldFromEntityInterface
     */
    private function getField()
    {
        return $this->field;
    }

    /**
     * @return Request
     */
    private function getRequest()
    {
        return $this->request;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->getField()->getName();
    }

    /**
     * @return string
     */
    public function getLabel()
    {
        return $this->getField()->getLabel();
    }

    /**
     * @return OptionInterface[]
     */
    public function getOptions()
    {
        return array_map(function (OptionInterface $option) {
            $selected = $this->isOptionSelected($option);
            return $option->withSelected($selected);
        }, $this->getField()->getOptions());
    }

    /**
     * @return int
     */
    public function getTypeId()
    {
        return $this->getField()->getTypeId();
    }

    /**
     * @return string
     */
    public function getValue()
    {
        return $this->getRequest()->get($this->getName());
    }

    /**
     * Indicate that the field was incorrectly filled
     *
     * @return bool
     */
    public function hasError()
    {
        return !$this->isValid();
    }

    /**
     * @return string
     */
    public function getError()
    {
        $errors = array_reduce(
            $this->getConstraints(),
            function (array $errors, CustomConstraintInterface $constraint) {
                if (!$constraint->isValid($this->getValue())) {
                    $errors[] = $constraint->getError();
                }

                return $errors;
            },
            []
        );

        return implode("<br>", $errors);
    }

    /**
     * The value of the type attribute for an input
     *
     * @return string|null
     */
    public function getInputType()
    {
        return $this->getField()->getInputType();
    }

    /**
     * @return string|null
     */
    public function getPattern()
    {
        return $this->getField()->getPattern();
    }

    /**
     * @return bool
     */
    public function isRequired()
    {
        return $this->getField()->isRequired();
    }

    /**
     * @return bool
     */
    public function isAutocompleteOff()
    {
        return $this->getField()->isAutocompleteOff();
    }

    /**
     * Indicates whether there should be an autofocus attribute
     *
     * @return bool
     */
    public function isFocused()
    {
        return $this->getField()->isFocused();
    }

    /**
     * @return bool
     */
    public function isDisabled()
    {
        return $this->getField()->isDisabled();
    }

    /**
     * @return bool
     */
    public function isReadOnly()
    {
        return $this->getField()->isReadOnly();
    }

    /**
     * @return string
     */
    public function getPlaceHolder()
    {
        return $this->getField()->getPlaceHolder();
    }

    /**
     * @return CustomConstraintInterface[]
     */
    public function getConstraints()
    {
        return $this->getFieldEntity()->getConstraints();
    }

    /**
     * @return string
     */
    public function getId()
    {
        return $this->getFieldEntity()->getId();
    }

    /**
     * @param OptionInterface $option
     * @return bool
     */
    private function isOptionSelected(OptionInterface $option)
    {
        $value = $this->getValue();
        if (is_array($value)) {
            $selected = in_array($option->getValue(), $value, true);
        } else {
            $selected = $value === $option->getValue();
        }
        return $selected;
    }

    /**
     * {@inheritdoc}
     */
    public function isDirty()
    {
        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function getTags()
    {
        return $this->getFieldEntity()->getTags();
    }
}
