<?php

namespace ByDedi\Module\CustomForm\Tests\Service;

use ByDedi\Module\CustomForm\Model\CustomConstraintInterface;
use ByDedi\Module\CustomForm\Model\FieldEntityInterface;
use ByDedi\Module\CustomForm\Service\FieldFromEntity;
use Form\Constraint\Callback;
use Form\Constraint\ConstraintInterface;
use Form\Constraint\EqualToField;
use Form\Constraint\Mail;
use Form\Constraint\NotBlank;
use Form\FullForm\FieldInterface;
use PHPUnit_Framework_TestCase;

class FieldFromEntityTest extends PHPUnit_Framework_TestCase
{
    public function testFieldIsCorrect()
    {
        $field = new FieldFromEntity('name_1', $this->getFieldEntity(
            'id1',
            FieldInterface::TYPE_TEXT_ID,
            'Le label',
            [
                $this->getConstraint(Mail::class, false),
                $this->getConstraint(EqualToField::class, true)
            ]
        ));

        $this->assertEquals('name_1', $field->getName());
        $this->assertEquals('Le label', $field->getLabel());
        $this->assertEquals(FieldInterface::TYPE_TEXT_ID, $field->getTypeId());
        $this->assertCount(2, $field->getConstraints());
        $this->assertFalse($field->isValid());
    }

    public function testIsRequired()
    {
        $field = new FieldFromEntity('name_1', $this->getFieldEntity(
            'id1',
            FieldInterface::TYPE_TEXT_ID,
            'Le label',
            [
                $this->getConstraint(Mail::class, true),
                $this->getConstraint(EqualToField::class, true)
            ]
        ));

        $this->assertFalse($field->isRequired());

        $field = new FieldFromEntity('name_2', $this->getFieldEntity(
            'id2',
            FieldInterface::TYPE_FILE_ID,
            'Fichier',
            [
                $this->getConstraint(Callback::class, false),
                $this->getConstraint(NotBlank::class, true)
            ]
        ));

        $this->assertTrue($field->isRequired());

        $field = new FieldFromEntity('name_3', $this->getFieldEntity(
            'id3',
            FieldInterface::TYPE_SELECT_ID,
            'select',
            [
                $this->getConstraint(CustomConstraintInterface::class, true),
                $this->getConstraint(NotBlank::class, true)
            ]
        ));

        $this->assertFalse($field->isRequired());
    }

    /**
     * @dataProvider inputTypeProvider
     *
     * @param $expectedInputType
     * @param ConstraintInterface[] ...$constraints
     */
    public function testHasCorrectInputType($expectedInputType, ConstraintInterface ...$constraints)
    {
        $field = new FieldFromEntity('name_3', $this->getFieldEntity(
            'id3',
            FieldInterface::TYPE_SELECT_ID,
            'select',
            $constraints
        ));

        $this->assertEquals($expectedInputType, $field->getInputType());
    }

    public function inputTypeProvider()
    {
        return [
            'Type par défaut' => [
                'text',
                $this->getConstraint(CustomConstraintInterface::class),
                $this->getConstraint(CustomConstraintInterface::class),
            ],
            'Type phone' => [
                'tel',
                $this->getConstraint(CustomConstraintInterface::class),
                $this->getConstraint(CustomConstraintInterface::class, false, false, true),
            ],
            'Type Mail' => [
                'email',
                $this->getConstraint(CustomConstraintInterface::class, false, false, true),
                $this->getConstraint(CustomConstraintInterface::class, false, true),
                $this->getConstraint(CustomConstraintInterface::class, false, false, true),
            ]
        ];
    }

    /**
     * @param $fieldId
     * @param $typeId
     * @param string $label
     * @param array $constraints
     * @return FieldEntityInterface
     */
    public function getFieldEntity($fieldId, $typeId, $label = '', array $constraints = [])
    {
        $formEntity = $this
            ->getMockBuilder(FieldEntityInterface::class)
            ->getMock()
        ;

        $formEntity
            ->method('getConstraints')
            ->willReturn($constraints)
        ;
        $formEntity
            ->method('getOptions')
            ->willReturn([])
        ;
        $formEntity
            ->method('getOptionByValue')
            ->willReturn(null)
        ;
        $formEntity
            ->method('getTypeId')
            ->willReturn($typeId)
        ;
        $formEntity
            ->method('getId')
            ->willReturn($fieldId)
        ;
        $formEntity
            ->method('getLabel')
            ->willReturn($label)
        ;

        return $formEntity;
    }

    /**
     * @param $constraintClass
     * @param bool $isValid
     * @param bool $isMail
     * @param bool $isPhone
     * @return ConstraintInterface
     */
    public function getConstraint($constraintClass, $isValid = false, $isMail = false, $isPhone = false)
    {
        $constraint = $this
            ->getMockBuilder($constraintClass)
            ->disableOriginalConstructor()
            ->getMock()
        ;
        $constraint
            ->method('isValid')
            ->willReturn($isValid)
        ;
        if ($constraint instanceof CustomConstraintInterface) {
            $constraint
                ->method('isMail')
                ->willReturn($isMail)
            ;

            $constraint
                ->method('isPhone')
                ->willReturn($isPhone)
            ;
        }

        return $constraint;
    }
}
